#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, sys, unittest, time
from ServiceTest import ServiceTestCase, ServiceTestSuite
from ZSI import FaultException
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.writer import SoapWriter
from ZSI.parse import ParsedSoap

"""
Unittest for Bug Report 
[ ] 

WSDL:  
"""

CONFIG_FILE = 'config.txt'
sys.path.append('%s/%s' %(os.getcwd(), 'stubs'))

class AttributeTest(unittest.TestCase):
    name = "test_Attributes"
    done = False

    def setUp(self):
        if AttributeTest.done: return
        AttributeTest.done = True
        wsdl = os.path.abspath('test_Attributes.xsd').strip()
        fin,fout = os.popen4('cd stubs; wsdl2py.py -x -b -f %s' %wsdl)
        # TODO: wait for wsdl2py.py to finish.
        for i in fout: print i
        import test_Attributes_xsd_services_types
 
    def tearDown(self):
        pass

    def test_attribute1(self):
        """
        """
        myDouble = 4.5
        myInt = 9
        myFloat = 3.0001
        myDecimal = 8.999
        myGDateTime = time.gmtime()
        myAnyURI = "urn:whatever"
        myQName = ("urn:test", "qbert")
        myString = "whatever"
        myHexBinary = hex(888)

        pyobj = GED("urn:example", "Test1").pyclass()
        # Test serialize/parse
        pyobj.set_attribute_myDecimal(myDecimal)
        pyobj.set_attribute_myDouble(myDouble)
        pyobj.set_attribute_myFloat(myFloat)
        pyobj.set_attribute_myInt(myInt)
        pyobj.set_attribute_myDateTime(myGDateTime)

        pyobj.set_attribute_myGDay(myGDateTime)
        pyobj.set_attribute_myGMonth(myGDateTime)
        pyobj.set_attribute_myGYear(myGDateTime)
        pyobj.set_attribute_myGYearMonth(myGDateTime)
        pyobj.set_attribute_myDate(myGDateTime)
        pyobj.set_attribute_myTime(myGDateTime)

        pyobj.set_attribute_myAnyURI(myAnyURI)
        pyobj.set_attribute_myString(myString)
        pyobj.set_attribute_myHexBinary(myHexBinary)
        pyobj.set_attribute_myDuration(myGDateTime)

        # Problems parsings 
        pyobj.set_attribute_myQName(myQName)
        pyobj.set_attribute_myGMonthDay(myGDateTime)


        #TODO:
        #pyobj.set_attribute_myBase64Binary("")
        #pyobj.set_attribute_myNOTATION("NOT")

        sw = SoapWriter()
        sw.serialize(pyobj)
        soap = str(sw)
 
        print soap
        ps = ParsedSoap(soap)
        pyobj2 = ps.Parse(pyobj.typecode)

        test = pyobj2.get_attribute_myInt()
        self.failUnlessEqual(myInt, test)

        test = pyobj2.get_attribute_myDouble()
        self.failUnlessEqual(myDouble, test)

        test = pyobj2.get_attribute_myFloat()
        self.failUnlessEqual(myFloat, test)

        test = pyobj2.get_attribute_myDecimal()
        self.failUnlessEqual(myDecimal, test)

        test = pyobj2.get_attribute_myAnyURI()
        self.failUnlessEqual(myAnyURI, test)

        test = pyobj2.get_attribute_myQName()
        self.failUnlessEqual(myQName, test)

        test = pyobj2.get_attribute_myString()
        self.failUnlessEqual(myString, test)

        test = pyobj2.get_attribute_myHexBinary()
        self.failUnlessEqual(myHexBinary, test)

        # DateTime stuff
        test = pyobj2.get_attribute_myDateTime()
        self.failUnlessEqual(myGDateTime[:-3], test[:-3])

        test = pyobj2.get_attribute_myDate()
        self.failUnlessEqual(myGDateTime[:3], test[:3])

        test = pyobj2.get_attribute_myTime()
        self.failUnlessEqual(myGDateTime[4:5], test[4:5])

        test = pyobj.get_attribute_myDuration()
        self.failUnlessEqual(myGDateTime, test)

        # Bug [ 1453421 ] Incorrect format for type gDay
        test = pyobj2.get_attribute_myGDay()
        self.failUnlessEqual(myGDateTime[2], test[2])

        test = pyobj2.get_attribute_myGMonth()
        self.failUnlessEqual(myGDateTime[1], test[1])

        test = pyobj2.get_attribute_myGYear()
        self.failUnlessEqual(myGDateTime[0], test[0])

        test = pyobj2.get_attribute_myGYearMonth()
        self.failUnlessEqual(myGDateTime[:2], test[:2])

        # hmm? negated?
        #test = pyobj2.get_attribute_myGMonthDay()
        #self.failUnlessEqual(myGDateTime[1:3], test[1:3])

    def test_empty_attribute(self):
        # [ 1452752 ] attribute with empty value doesn't appear in parsed object
        myString = ""
        pyobj = GED("urn:example", "Test1").pyclass()
        pyobj.set_attribute_myString(myString)

        sw = SoapWriter()
        sw.serialize(pyobj)
        soap = str(sw)
 
        print soap
        ps = ParsedSoap(soap)
        pyobj2 = ps.Parse(pyobj.typecode)

        test = pyobj2.get_attribute_myString() 
        self.failUnlessEqual(myString, str(test))


def makeTestSuite():
    suite = ServiceTestSuite()
    suite.addTest(unittest.makeSuite(AttributeTest, "test_", suiteClass=ServiceTestSuite))
    return suite


if __name__ == "__main__" :
    unittest.TestProgram(defaultTest="makeTestSuite")




#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys, unittest
from ServiceTest import ServiceTestCase, ServiceTestSuite
"""
Unittest for contacting the Map Point Service.  

WSDL:  
"""

CONFIG_FILE = 'config.txt'
SERVICE_NAME = 'CommonService'
PORT_NAME = 'CommonServiceSoap'


class MapPointTest(ServiceTestCase):
    """Test case for OPCService Web service
    
    """
    name = "test_MapPoint"

    def setUp(self):
        ServiceTestCase.setSection(self,self.name)
        ServiceTestCase.setUp(self)

    def _getLocator(self):
        """Multiple services, so grab the locator you want
        """
        locatorName = self._wsm.services[0].locator.getLocatorName()
        if self._moduleDict.has_key(self._serviceModuleName):
            sm = self._moduleDict[self._serviceModuleName]
            if sm.__dict__.has_key(locatorName):
                return sm.__dict__[locatorName]()
            raise TestException, 'No Locator class (%s), missing SOAP location binding?' %locatorName
        raise TestException, 'missing service module, possibly no WSDL service item?'

    def _getPort(self, locator, netloc=None):
        """Returns a rpc proxy port instance.
           **Only expecting 1 service/WSDL, and 1 port/service.

           locator -- Locator instance
        """
        #if len(self._wsm.services) != 1:
        #    raise TestException, 'only supporting WSDL with one service information item'
        methodNames = self._wsm.services[0].locator.getPortMethods()
        if len(methodNames) == 0:
            raise TestException, 'No port defined for service.'
        #elif len(methodNames) > 1:
        #    raise TestException, 'Not handling multiple ports.'
        callMethod = getattr(locator, methodNames[0])
        return callMethod(**self._getPortOptions(methodNames[0], locator, netloc))
 
    def test_GetVersionInfo(self):
        """expect this to fail cause i'm not doing http authentication.
        """
        operationName = 'GetVersionInfo'
        msg = self.getInputMessageInstance(operationName)
        try:
            response = self.RPC(operationName, msg)
        except:
            pass
 
    

def makeTestSuite():
    suite = ServiceTestSuite()
    suite.addTest(unittest.makeSuite(MapPointTest, "test_", suiteClass=ServiceTestSuite))
    return suite


if __name__ == "__main__" :
    unittest.TestProgram(defaultTest="makeTestSuite")




#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, sys, unittest
from ServiceTest import ServiceTestCase, ServiceTestSuite
from ZSI import FaultException
"""
Unittest 

WSDL:  
"""

CONFIG_FILE = 'config.txt'
SERVICE_NAME = 'EchoServer'
PORT_NAME = 'EchoServer'

sys.path.append('%s/%s' %(os.getcwd(), 'stubs'))

class EchoTest(unittest.TestCase):
    name = "test_Echo"
    done = False

    def setUp(self):
        if EchoTest.done: return
        EchoTest.done = True
        wsdl = os.path.abspath('../../samples/Echo/Echo.wsdl').strip()
        fin,fout = os.popen4('cd stubs; wsdl2py.py -f %s' %wsdl)
        # TODO: wait for wsdl2py.py to finish.
        for i in fout: print i
 
    def tearDown(self):
        pass

    def test_Echo(self):
        from EchoServer_services import EchoServerLocator, EchoRequest, EchoResponse
        port = EchoServerLocator().getEchoServer()
    

def makeTestSuite():
    suite = ServiceTestSuite()
    suite.addTest(unittest.makeSuite(EchoTest, "test_", suiteClass=ServiceTestSuite))
    return suite


if __name__ == "__main__" :
    unittest.TestProgram(defaultTest="makeTestSuite")




###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
# $Id: WSresource.py 1155 2006-03-15 18:12:09 -0500 (Wed, 15 Mar 2006) boverhof $
###########################################################################

import sys, warnings

# twisted & related imports
from zope.interface import classProvides, implements, Interface
from twisted.python import log, failure
from twisted.web.error import NoResource
from twisted.web.server import NOT_DONE_YET
import twisted.web.http
import twisted.web.resource

# ZSI imports
from ZSI import _get_element_nsuri_name, EvaluateException, ParseException
from ZSI.parse import ParsedSoap
from ZSI.writer import SoapWriter
from ZSI import fault

# WS-Address related imports
from ZSI.address import Address
from ZSI.ServiceContainer import WSActionException


# 
# Stability: Unstable
# 

class HandlerChainInterface(Interface):
    
    def processRequest(self, input, **kw):
        """returns a representation of the request, the 
        last link in the chain must return a response
        pyobj with a typecode attribute.
        Parameters:
            input --
        Keyword Parameters:
            request -- HTTPRequest instance
            resource  -- Resource instance
        """
    def processResponse(self, output, **kw):
        """returns a string representing the soap response.
        Parameters
            output --
        Keyword Parameters:
            request -- HTTPRequest instance
            resource  -- Resource instance
        """

class CallbackChainInterface(Interface):
    
    def processRequest(self, input, **kw):
        """returns a response pyobj with a typecode 
        attribute.
        Parameters:
            input --
        Keyword Parameters:
            request -- HTTPRequest instance
            resource  -- Resource instance
        """

class DataHandler:
    """
    class variables:
        readerClass -- factory class to create reader for ParsedSoap instances.
        writerClass -- ElementProxy implementation to use for SoapWriter instances.
    """
    classProvides(HandlerChainInterface)
    readerClass = None
    writerClass = None

    @classmethod
    def processRequest(cls, input, **kw):
        return ParsedSoap(input, readerclass=cls.readerClass)

    @classmethod
    def processResponse(cls, output, **kw):
        sw = SoapWriter(outputclass=cls.writerClass)
        sw.serialize(output)
        return sw
    
    
class DefaultCallbackHandler:
    classProvides(CallbackChainInterface)

    @classmethod
    def processRequest(cls, ps, **kw):
        """invokes callback that should return a (request,response) tuple.
        representing the SOAP request and response respectively.
        ps -- ParsedSoap instance representing HTTP Body.
        request -- twisted.web.server.Request
        """
        resource = kw['resource']
        request = kw['request']
        method =  getattr(resource, 'soap_%s' %
                           _get_element_nsuri_name(ps.body_root)[-1])
                                              
        try:
            req_pyobj,rsp_pyobj = method(ps, request=request)
        except TypeError, ex:
            log.err(
                'ERROR: service %s is broken, method MUST return request, response'\
                    % cls.__name__
            )
            raise
        except Exception, ex:
            log.err('failure when calling bound method')
            raise
        
        return rsp_pyobj
    

class WSAddressHandler:
    """General WS-Address handler.  This implementation depends on a 
    'wsAction' dictionary in the service stub which contains keys to 
    WS-Action values.  

    Implementation saves state on request response flow, so using this 
    handle is not  reliable if execution is deferred between proceesRequest 
    and processResponse.  

    TODO: sink this up with wsdl2dispatch
    TODO: reduce coupling with WSAddressCallbackHandler.
    """
    implements(HandlerChainInterface)
    
    def processRequest(self, ps, **kw):
        # TODO: Clean this up
        resource = kw['resource']

        d = getattr(resource, 'root', None)
        key = _get_element_nsuri_name(ps.body_root)
        if d is None or d.has_key(key) is False:
            raise RuntimeError,\
                'Error looking for key(%s) in root dictionary(%s)' %(key, str(d))

        self.op_name = d[key]
        self.address = address = Address()
        address.parse(ps)
        action = address.getAction()
        if not action:
            raise WSActionException('No WS-Action specified in Request')

        request = kw['request']
        http_headers = request.getAllHeaders()
        soap_action = http_headers.get('soapaction')
        if soap_action and soap_action.strip('\'"') != action:
            raise WSActionException(\
                'SOAP Action("%s") must match WS-Action("%s") if specified.'\
                %(soap_action,action)
            )
            
        # Save WS-Address in ParsedSoap instance.
        ps.address = address
        return ps
        
    def processResponse(self, sw, **kw):
        if sw is None:
            self.address = None
            return
        
        request, resource = kw['request'], kw['resource']
        if isinstance(request, twisted.web.http.Request) is False:
            raise TypeError, '%s instance expected' %http.Request
                
        d = getattr(resource, 'wsAction', None)
        key = self.op_name
        if d is None or d.has_key(key) is False:
            raise WSActionNotSpecified,\
                'Error looking for key(%s) in wsAction dictionary(%s)' %(key, str(d))

        addressRsp = Address(action=d[key])
        if request.transport.TLS == 0:
            addressRsp.setResponseFromWSAddress(\
                 self.address, 'http://%s:%d%s' %(
                 request.host.host, request.host.port, request.path)
            )
        else:
            addressRsp.setResponseFromWSAddress(\
                 self.address, 'https://%s:%d%s' %(
                 request.host.host, request.host.port, request.path)
            )
            
        addressRsp.serialize(sw, typed=False)
        self.address = None
        return sw
    

class WSAddressCallbackHandler:
    classProvides(CallbackChainInterface)

    @classmethod
    def processRequest(cls, ps, **kw):
        """invokes callback that should return a (request,response) tuple.
        representing the SOAP request and response respectively.
        ps -- ParsedSoap instance representing HTTP Body.
        request -- twisted.web.server.Request
        """
        resource = kw['resource']
        request = kw['request']
        method =  getattr(resource, 'wsa_%s' %
                           _get_element_nsuri_name(ps.body_root)[-1])
                                              
        # TODO: grab ps.address, clean this up.
        try:
            req_pyobj,rsp_pyobj = method(ps, ps.address, request=request)
        except TypeError, ex:
            log.err(
                'ERROR: service %s is broken, method MUST return request, response'\
                    %self.__class__.__name__
            )
            raise
        except Exception, ex:
            log.err('failure when calling bound method')
            raise
        
        return rsp_pyobj


def CheckInputArgs(*interfaces):
    """Must provide at least one interface, the last one may be repeated.
    """
    l = len(interfaces)
    def wrapper(func):
        def check_args(self, *args, **kw):
            for i in range(len(args)):
                if (l > i and interfaces[i].providedBy(args[i])) or interfaces[-1].providedBy(args[i]):
                    continue
                if l > i: raise TypeError, 'arg %s does not implement %s' %(args[i], interfaces[i])
                raise TypeError, 'arg %s does not implement %s' %(args[i], interfaces[-1])
            func(self, *args, **kw)
        return check_args
    return wrapper
            

class DefaultHandlerChain:

    @CheckInputArgs(CallbackChainInterface, HandlerChainInterface)
    def __init__(self, cb, *handlers):
        self.handlercb = cb
        self.handlers = handlers
        self.debug = len(log.theLogPublisher.observers) > 0
        
    def processRequest(self, arg, **kw):
        if self.debug:
            log.msg('--->PROCESS REQUEST\n%s' %arg, debug=1)

        for h in self.handlers:
            arg = h.processRequest(arg, **kw)
            
        return self.handlercb.processRequest(arg, **kw)
            
    def processResponse(self, arg, **kw):
        if self.debug:
            log.msg('===>PROCESS RESPONSE: %s' %str(arg), debug=1)

        if arg is None: 
            return

        for h in self.handlers:
            arg = h.processResponse(arg, **kw)
            
        s = str(arg)
        if self.debug:
            log.msg(s, debug=1)

        return s


class DefaultHandlerChainFactory:
    protocol = DefaultHandlerChain
    
    @classmethod
    def newInstance(cls):
        return cls.protocol(DefaultCallbackHandler, DataHandler)
    

class WSAddressHandlerChainFactory:
    protocol = DefaultHandlerChain
    
    @classmethod
    def newInstance(cls):
        return cls.protocol(WSAddressCallbackHandler, DataHandler, 
            WSAddressHandler())
    

class WSResource(twisted.web.resource.Resource, object):
    """
    class variables:
        encoding  --
        factory -- hander chain, which has a factory method "newInstance"
		that returns a 
    """
    encoding = "UTF-8"
    factory = DefaultHandlerChainFactory

    def __init__(self):
        """
        chain -- request/response handler chain.
        """
        self.chain = self.factory.newInstance()
        twisted.web.resource.Resource.__init__(self)

    def _writeResponse(self, request, response, status=200):
        """
        request -- request message
        response --- response message
        status -- HTTP Status
        """
        request.setResponseCode(status)
        if self.encoding is not None:
            mimeType = 'text/xml; charset="%s"' % self.encoding
        else:
            mimeType = "text/xml"

        request.setHeader("Content-type", mimeType)
        request.setHeader("Content-length", str(len(response)))
        request.write(response)
        request.finish()
        return NOT_DONE_YET

    def _writeFault(self, request, ex):
        """
        request -- request message
        ex -- Exception 
        """
        response = None
        response = fault.FaultFromException(ex, False, sys.exc_info()[2]).AsSOAP()
        log.err('SOAP FAULT: %s' % response)
        return self._writeResponse(request, response, status=500)

    def render_POST(self, request):
        """Dispatch Method called by twisted render.
        request -- twisted.web.server.Request
        """
        data = request.content.read()
        try:
            pyobj = self.chain.processRequest(data, request=request, resource=self)
        except Exception, ex:
            return self._writeFault(request, ex)

        try:
            soap = self.chain.processResponse(pyobj, request=request, resource=self)
        except Exception, ex:
            return self._writeFault(request, ex)

        if soap is not None:
            return self._writeResponse(request, soap)

        request.finish()
        return NOT_DONE_YET




################################################## 
# test_Choice_xsd_services_types.py 
# generated by ZSI.generate.wsdl2python
##################################################


import ZSI
import ZSI.TCcompound
from ZSI.TC import ElementDeclaration,TypeDefinition
from ZSI.generate.pyclass import pyclass_type

##############################
# targetNamespace
# urn:example
##############################

class ns0:
    targetNamespace = "urn:example"

    class EasyChoice_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "urn:example"
        type = (schema, "EasyChoice")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.EasyChoice_Def.schema
            TClist = [ZSI.TC.String(pname="name", aname="_name", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TCnumbers.Iint(pname="rank", aname="_rank", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.AnyType(pname="any", aname="_any", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._name = None
                    self._rank = None
                    self._any = None
                    return
            Holder.__name__ = "EasyChoice_Holder"
            self.pyclass = Holder

    class HardChoice_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "urn:example"
        type = (schema, "HardChoice")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.HardChoice_Def.schema
            TClist = [ZSI.TC.String(pname="name", aname="_name", minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded")), ZSI.TCnumbers.Iint(pname="rank", aname="_rank", minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded")), ZSI.TC.AnyType(pname="any", aname="_any", minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._name = []
                    self._rank = []
                    self._any = []
                    return
            Holder.__name__ = "HardChoice_Holder"
            self.pyclass = Holder

    class Easy_Dec(ElementDeclaration):
        literal = "Easy"
        schema = "urn:example"
        def __init__(self, **kw):
            kw["pname"] = ("urn:example","Easy")
            kw["aname"] = "_Easy"
            if ns0.EasyChoice_Def not in ns0.Easy_Dec.__bases__:
                bases = list(ns0.Easy_Dec.__bases__)
                bases.insert(0, ns0.EasyChoice_Def)
                ns0.Easy_Dec.__bases__ = tuple(bases)

            ns0.EasyChoice_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "Easy_Dec_Holder"

    class Hard_Dec(ElementDeclaration):
        literal = "Hard"
        schema = "urn:example"
        def __init__(self, **kw):
            kw["pname"] = ("urn:example","Hard")
            kw["aname"] = "_Hard"
            if ns0.HardChoice_Def not in ns0.Hard_Dec.__bases__:
                bases = list(ns0.Hard_Dec.__bases__)
                bases.insert(0, ns0.HardChoice_Def)
                ns0.Hard_Dec.__bases__ = tuple(bases)

            ns0.HardChoice_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "Hard_Dec_Holder"

# end class ns0 (tns: urn:example)



################################################## 
# RPC_Literal_TestDefinitions_services_types.py 
# generated by ZSI.generate.wsdl2python
##################################################


import ZSI
import ZSI.TCcompound
from ZSI.TC import ElementDeclaration,TypeDefinition
from ZSI.generate.pyclass import pyclass_type

##############################
# targetNamespace
# http://soapbuilders.org/rpc-lit-test/types
##############################

class ns0:
    targetNamespace = "http://soapbuilders.org/rpc-lit-test/types"

    class ArrayOfstring_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "ArrayOfstring")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ArrayOfstring_Def.schema
            TClist = [ns0.stringItem_Dec(minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._stringItem = None
                    return
            Holder.__name__ = "ArrayOfstring_Holder"
            self.pyclass = Holder

    class ArrayOfint_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "ArrayOfint")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ArrayOfint_Def.schema
            TClist = [ZSI.TCnumbers.Iint(pname="integer", aname="_integer", minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._integer = []
                    return
            Holder.__name__ = "ArrayOfint_Holder"
            self.pyclass = Holder

    class SOAPStruct_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "SOAPStruct")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.SOAPStruct_Def.schema
            TClist = []
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    return
            Holder.__name__ = "SOAPStruct_Holder"
            self.pyclass = Holder

    class ArrayOfSOAPStruct_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "ArrayOfSOAPStruct")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ArrayOfSOAPStruct_Def.schema
            TClist = [ns0.structItem_Dec(minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._structItem = None
                    return
            Holder.__name__ = "ArrayOfSOAPStruct_Holder"
            self.pyclass = Holder

    class SOAPStructStruct_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "SOAPStructStruct")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.SOAPStructStruct_Def.schema
            TClist = []
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    return
            Holder.__name__ = "SOAPStructStruct_Holder"
            self.pyclass = Holder

    class SOAPArrayStruct_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        type = (schema, "SOAPArrayStruct")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.SOAPArrayStruct_Def.schema
            TClist = []
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    return
            Holder.__name__ = "SOAPArrayStruct_Holder"
            self.pyclass = Holder

    class stringItem_Dec(ZSI.TC.String, ElementDeclaration):
        literal = "stringItem"
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        def __init__(self, **kw):
            kw["pname"] = ("http://soapbuilders.org/rpc-lit-test/types","stringItem")
            kw["aname"] = "_stringItem"
            class IHolder(str): typecode=self
            kw["pyclass"] = IHolder
            IHolder.__name__ = "_stringItem_immutable_holder"
            ZSI.TC.String.__init__(self, **kw)

    class structItem_Dec(ElementDeclaration):
        literal = "structItem"
        schema = "http://soapbuilders.org/rpc-lit-test/types"
        def __init__(self, **kw):
            kw["pname"] = ("http://soapbuilders.org/rpc-lit-test/types","structItem")
            kw["aname"] = "_structItem"
            if ns0.SOAPStruct_Def not in ns0.structItem_Dec.__bases__:
                bases = list(ns0.structItem_Dec.__bases__)
                bases.insert(0, ns0.SOAPStruct_Def)
                ns0.structItem_Dec.__bases__ = tuple(bases)

            ns0.SOAPStruct_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "structItem_Dec_Holder"

# end class ns0 (tns: http://soapbuilders.org/rpc-lit-test/types)



#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################

import os, sys, unittest, urlparse, signal, time, warnings
from ConfigParser import ConfigParser, NoSectionError, NoOptionError
"""Global Variables:
    CONFIG_FILE -- configuration file 
    CONFIG_PARSER -- ConfigParser instance
    DOCUMENT -- test section variable, specifying document style.
    LITERAL -- test section variable, specifying literal encodings.
    BROKE -- test section variable, specifying broken test.
    TESTS -- test section variable, whitespace separated list of modules.
    SECTION_CONFIGURATION -- configuration section, turn on/off debuggging.
    TRACEFILE -- file class instance.
    TOPDIR -- current working directory
    MODULEDIR  -- stubs directory 
    PORT -- port of local container
    HOST -- address of local container
    SECTION_SERVERS -- services to be tested, values are paths to executables.
"""
CONFIG_FILE = 'config.txt'
CONFIG_PARSER = ConfigParser()
DOCUMENT = 'document'
LITERAL = 'literal'
BROKE = 'broke'
TESTS = 'tests'
SECTION_CONFIGURATION = 'configuration'
TRACEFILE = sys.stdout
TOPDIR = os.getcwd()
MODULEDIR = TOPDIR + '/stubs'
PORT = 'port'
HOST = 'host'
SECTION_SERVERS = 'servers'

CONFIG_PARSER.read(CONFIG_FILE)
if CONFIG_PARSER.getboolean(SECTION_CONFIGURATION, 'debug'):
    from ZSI.wstools.logging import setBasicLoggerDEBUG
    setBasicLoggerDEBUG()


from ZSI.wstools import WSDLTools
from ZSI.wstools.Namespaces import WSA200408, WSA200403, WSA200303
from ZSI.wstools.TimeoutSocket import TimeoutError
from ZSI.generate.wsdl2python import WriteServiceModule
from ZSI.generate.wsdl2dispatch import ServiceModuleWriter, WSAServiceModuleWriter
import StringIO, copy, getopt

# set up pyclass metaclass for complexTypes
from ZSI.generate.containers import TypecodeContainerBase, TypesHeaderContainer
TypecodeContainerBase.metaclass = 'pyclass_type'
TypesHeaderContainer.imports.append(\
        'from ZSI.generate.pyclass import pyclass_type'
        )


sys.path.append('%s/%s' %(os.getcwd(), 'stubs'))
print sys.path[-1]
ENVIRON = copy.copy(os.environ)
ENVIRON['PYTHONPATH'] = ENVIRON.get('PYTHONPATH', '') + ':' + MODULEDIR


def WriteServiceStubModule(wsdl):
    '''Create/Write Service Skeleton module.  If WS-Addressing schema is present 
    assume this is a WS-Address enabled service.
    '''
    if len(filter(lambda addr: wsdl.types.has_key(addr), (WSA200408.ADDRESS, WSA200403.ADDRESS, WSA200303.ADDRESS))) > 0:
        ss = WSAServiceModuleWriter
    else:
        ss = ServiceModuleWriter()
    ss.fromWSDL(wsdl)
    module_name = ss.getServiceModuleName()
    fd = open(module_name + '.py', 'w+')
    ss.write(fd)
    fd.close()
    return module_name


def SpawnContainer(ex_name):
    '''module_name must be executable, and set up ServiceContainer
    to test against.
    ex -- executable name
    '''
    port = CONFIG_PARSER.get(SECTION_CONFIGURATION, PORT)
    processID = os.spawnle(os.P_NOWAIT, ex_name, ex_name, port, ENVIRON)
    time.sleep(1)
    return processID


class ConfigException(Exception):
    """Exception thrown when configuration settings arent correct.
    """
    pass

class TestException(Exception):
    """Exception thrown when test case isn't correctly set up.
    """
    pass


class ServiceTestCase(unittest.TestCase):
    """Must set these class variables typeModuleName, serviceModuleName,
       path, and schema.  moduleDict is filled in after modules are created.

       class variables:
           name -- configuration item, must be set in class.
           url_section -- configuration section, maps a test module name to an URL.
    """
    name = None
    url_section = 'WSDL'

    def __init__(self, methodName):
        """
        parameters:
           methodName -- 
        instance variables:
           _moduleDict -- module dictionary
           _ports -- soap ports (must be same binding), proxy rpc instance, to run tests on.
           _portName -- The name of the port to get using _getPort
           _importTypeModule -- boolean indicating whether to import the types file 
           _typeModuleName -- key to the generated type module
           _serviceModuleName -- key to the generated service module
           _section -- if set, do a runtime check that test is correctly indexed.
           _wsm -- WriteServiceModule instance
        """
        self._moduleDict = {}
        self._ports = None
        self._portName = None
        self._importTypeModule = True
        self._typeModuleName = None
        self._serviceModuleName = None
        self._wsm = None
        self._section = None
        self._processID = None
        unittest.TestCase.__init__(self, methodName)

    def setUp(self):
        """Generate types and services modules if no record of them
           in the moduleDict variable.
        """
        
        if not self.url_section or not self.name:
            raise TestException, 'section(%s) or name(%s) not defined' %(self.url_section,self.name)
        if self._isSetModules():
            return
        if not CONFIG_PARSER.has_section(self.url_section):
            raise TestException, 'No such section(%s) in configuration file(%s)' \
                %(self.url_section, CONFIG_FILE)

        #try:
        #    os.mkdir(MODULEDIR)
        #except OSError, ex:
        #    pass

        #os.chdir(MODULEDIR)
        #if MODULEDIR not in sys.path:
        #    sys.path.append(MODULEDIR)

        reader = WSDLTools.WSDLReader()
        url = CONFIG_PARSER.get(self.url_section, self.name)

        try:
            wsdl = reader.loadFromURL(url)
        except TimeoutError:
            # SKIP 
            self.fail('socket timeout retrieving WSDL: %s' %url)
            #os.chdir(TOPDIR)
        except:
            #os.chdir(TOPDIR)
            raise

        try:
            os.mkdir(MODULEDIR)
        except OSError, ex:
            pass

        os.chdir(MODULEDIR)
        if MODULEDIR not in sys.path:
            sys.path.append(MODULEDIR)
           
        try:
            self._wsm = WriteServiceModule(wsdl)
            fd = open('%s.py' %self._wsm.getTypesModuleName(), 'w+')
            self._wsm.writeTypes(fd)
            fd.close()
            fd = open('%s.py' %self._wsm.getClientModuleName(), 'w+')
            self._wsm.writeClient(fd)
            fd.close()
            self._setModuleNames(self._importTypeModule)
            module_name = WriteServiceStubModule(wsdl)
        finally:
            os.chdir(TOPDIR)

        locator = self._getLocator()
        self._ports = []
        if CONFIG_PARSER.getboolean(SECTION_CONFIGURATION, 'net'):
            self._ports.append(self._getPort(locator))

        if CONFIG_PARSER.getboolean(SECTION_CONFIGURATION, 'local'):
            netloc = '%s:%d' %(CONFIG_PARSER.get(SECTION_CONFIGURATION, HOST),
                CONFIG_PARSER.getint(SECTION_CONFIGURATION, PORT))
            try:
                ex_path = CONFIG_PARSER.get(SECTION_SERVERS, self.name)
            except (NoSectionError, NoOptionError), ex:
                warnings.warn('skipping local test for %s' %self.name)
            else:
                self._ports.append(self._getPort(locator, netloc=netloc))
                self._processID = SpawnContainer(TOPDIR + '/' + ex_path)

    def tearDown(self):
        """
        This must be deleted because it has reference to
        a namespace hash object which keeps its dictionary in
        a class variable.If we run multiple ServiceTestSuites then
        namespaces in the first prior ServiceTestSuite will remain
        in the hash, and the code generator will import them in the
        We need a fresh namespace hash each time a unittest is run
        """
        del self._wsm
        if self._processID is not None:
            os.kill(self._processID, signal.SIGKILL)
        
    def _getPortOptions(self, methodName, locator, netloc=None):
        kw = {}
        if CONFIG_PARSER.getboolean(SECTION_CONFIGURATION, 'tracefile'):
            kw['tracefile'] = TRACEFILE

        if netloc is not None:
            default_url = getattr(locator, methodName + 'Address')()
            scheme, netloc_old, url, params, query, fragment = urlparse.urlparse(default_url)
            kw['url'] = urlparse.urlunparse((scheme, netloc, url, params, query, fragment))

        return kw

    def _getLocator(self):
        """Get the Locator name
           **Only expecting 1 service/WSDL, and 1 port/service.
           wsm -- WriteServiceModule instance
        """
        if len(self._wsm.services) != 1:
            raise TestException, 'only supporting WSDL with one service information item'
        locatorName = self._wsm.services[0].locator.getLocatorName()
        if self._moduleDict.has_key(self._serviceModuleName):
            sm = self._moduleDict[self._serviceModuleName]
            if sm.__dict__.has_key(locatorName):
                return sm.__dict__[locatorName]()
            raise TestException, 'No Locator class (%s), missing SOAP location binding?' %locatorName
        raise TestException, 'missing service module, possibly no WSDL service item?'

    def _getPort(self, locator, netloc=None):
        """Returns a rpc proxy port instance.
           **Only expecting 1 service/WSDL, and 1 port/service.

           locator -- Locator instance
        """
        if len(self._wsm.services) != 1:
            raise TestException, 'only supporting WSDL with one service information item'
        methodNames = self._wsm.services[0].locator.getPortMethods()
        if len(methodNames) == 0: 
            raise TestException, 'No port defined for service.'
        elif len(methodNames) > 1:
            raise TestException, 'Not handling multiple ports.'
        callMethod = getattr(locator, methodNames[0])
        return callMethod(**self._getPortOptions(methodNames[0], locator, netloc))

    def _isSetModules(self):
        """Are the types/service modules already loaded?
        """
        if self._serviceModuleName: 
            return True
        return False

    def _setModuleNames(self, importTypes=True):
        """set service and types modules key names, and import them.

           Some of the no schema tests do generate a types file
        """
        self._typeModuleName = self._wsm.getTypesModuleName()
        self._serviceModuleName = self._wsm.getClientModuleName()
        if importTypes:
            moduleTuple = (self._typeModuleName, self._serviceModuleName)
        else:
            moduleTuple = (self._serviceModuleName,)

        for name in moduleTuple:
            exec('import %s' %name)
            self._moduleDict[name] = eval(name)


    def checkSection(self):
        """Should the section be checked? If I know about it.
        """
        return type(self._section) is str

    def _checkPort(self, port, name):
        """Check if test is in correctly indexed in the configuration file, 
           port operation must conform to the bindings.  

           We check the style and use attributes from the soapOperation element.
           
           port -- WSDLTools.Port instance
           name -- WSDL port operation name
           port -- WSDLTools.Port instance
        """
        if self.checkSection() is False:
            return

        doc = CONFIG_PARSER.getboolean(self.getSection(), DOCUMENT)
        lit = CONFIG_PARSER.getboolean(self.getSection(), LITERAL)
        binding = port.getBinding()
        if binding.operations.has_key(name) is False:
            raise TestException, 'port(%s) binding(%s) has no operation(%s)'\
                %(port.name, port.type, name)

        soapbinding = binding.findBinding(WSDLTools.SoapBinding)
        if soapbinding is None:
            raise TestException, 'Missing soap:binding element'

        opbinding = binding.operations[name]
        if opbinding.input is None:
            raise TestException, 'wsdl:binding(%s) operation(%s) is missing input' \
                %(binding.name, name)
       
        msgrole = opbinding.input
        soap_op_binding = opbinding.findBinding(WSDLTools.SoapOperationBinding)
        style = soap_op_binding.style or soapbinding.style
        use = None
        body = msgrole.findBinding(WSDLTools.SoapBodyBinding)
        if body is None:
            raise TestException, 'Missing soap:body element'

        use = body.use
        style = soap_op_binding.style or soapbinding.style
        if doc and style != 'document':
            raise ConfigException, 'operation(%s) is not document style' %name
        elif not doc and style != 'rpc':
            raise ConfigException, 'operation(%s) is not rpc style' %name
        if lit is True and use != 'literal':
            raise ConfigException, 'operation(%s) is not literal encoding' %name
        if lit is False and use != 'encoded':
            raise ConfigException, 'operation(%s) is not rpc style' %name
    
    def setSection(self, section):
        """section -- this is the section the service is categorized in,
               if set STC will do a sanity check against the WSDL, etc.
        """
        self._section = section 

    def getSection(self):
        return self._section

    def getInputMessageInstance(self, operationName):
        """Returns an instance of the input message to send for this operation.
           Assuming there is only 1 service/wsdl, and 1 port/service.

           operationName -- WSDL port operation name
        """
        service = self._wsm._wsdl.services[0]
        port = service.ports[0]
        try:
            self._checkPort(port, operationName)
        except TestException, ex:
            raise TestException('Service(%s)' %service.name, *ex.args)

        for operation in port.getBinding().getPortType().operations:
            if operation.name == operationName: 
                break
        else:
            raise TestException, 'Missing operation (%s) in portType operations'  %operationName

        inputMsgName = '%s' %operation.input.message[1]
        if self._moduleDict.has_key(self._serviceModuleName):
            sm = self._moduleDict[self._serviceModuleName]
            if sm.__dict__.has_key(inputMsgName):
                inputMsgWrapper = sm.__dict__[inputMsgName]()
                return inputMsgWrapper
            else:
                raise TestException, 'service missing input message (%s)' %(inputMsgName)

        raise TestException, 'port(%s) does not define operation %s' \
            %(port.name, operationName)

    def RPC(self, operationName, request):
        """RPC serializes request, and returns response.

           operationName -- WSDL port operation name
        """
        for port in self._ports:
            callMethod = getattr(port, operationName, None)
            if callMethod is not None:
                callMethod(request)
            else:
                raise TestException, 'Port instance is missing method %s' %operationName 


class ServiceTestSuite(unittest.TestSuite):
    """A test suite is a composite test consisting of a number of TestCases.

    For use, create an instance of TestSuite, then add test case instances.
    When all tests have been added, the suite can be passed to a test
    runner, such as TextTestRunner. It will run the individual test cases
    in the order in which they were added, aggregating the results. When
    subclassing, do not forget to call the base class constructor.
    """
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests)
        self._section = None

    def setSection(self, section):
        self._section = section

    def addTest(self, test):
        if isinstance(test, ServiceTestSuite):
            test.setSection(self._section)
        unittest.TestSuite.addTest(self, test)




################################################## 
# SessionCreateRQService_services.py 
# generated by ZSI.generate.wsdl2python
##################################################


from SessionCreateRQService_services_types import *
import urlparse, types
from ZSI.TCcompound import ComplexType, Struct
from ZSI import client
import ZSI

# Locator
class SessionCreateRQServiceLocator:
    SessionCreatePortType_address = "https://webservices.sabre.com/websvc"
    def getSessionCreatePortTypeAddress(self):
        return SessionCreateRQServiceLocator.SessionCreatePortType_address
    def getSessionCreatePortType(self, url=None, **kw):
        return SessionCreateSoapBindingSOAP(url or SessionCreateRQServiceLocator.SessionCreatePortType_address, **kw)

# Methods
class SessionCreateSoapBindingSOAP:
    def __init__(self, url, **kw):
        kw.setdefault("readerclass", None)
        kw.setdefault("writerclass", None)
        # no resource properties
        self.binding = client.Binding(url=url, **kw)
        # no ws-addressing

    # op: SessionCreateRQ
    def SessionCreateRQ(self, request):
        if isinstance(request, GetSessionCreateInput) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="OTA", **kw)
        # no output wsaction
        response = self.binding.Receive(GetSessionCreateOutput.typecode)
        if isinstance(response, GetSessionCreateOutput.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

GetSessionCreateInput = ns4.SessionCreateRQ_Dec().pyclass

GetSessionCreateOutput = ns4.SessionCreateRS_Dec().pyclass



################################################## 
# ZipCodeResolver_services_types.py 
# generated by ZSI.generate.wsdl2python
##################################################


import ZSI
import ZSI.TCcompound
from ZSI.TC import ElementDeclaration,TypeDefinition
from ZSI.generate.pyclass import pyclass_type

##############################
# targetNamespace
# http://webservices.eraserver.net/
##############################

class ns1:
    targetNamespace = "http://webservices.eraserver.net/"

    class USPSAddress_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://webservices.eraserver.net/"
        type = (schema, "USPSAddress")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns1.USPSAddress_Def.schema
            TClist = [ZSI.TC.String(pname=(ns,"Street"), aname="_Street", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"City"), aname="_City", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"State"), aname="_State", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"ShortZIP"), aname="_ShortZIP", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"FullZIP"), aname="_FullZIP", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._Street = None
                    self._City = None
                    self._State = None
                    self._ShortZIP = None
                    self._FullZIP = None
                    return
            Holder.__name__ = "USPSAddress_Holder"
            self.pyclass = Holder

    class FullZipCode_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "FullZipCode"
        def __init__(self, **kw):
            ns = ns1.FullZipCode_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"accessCode"), aname="_accessCode", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"address"), aname="_address", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"city"), aname="_city", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"state"), aname="_state", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","FullZipCode")
            kw["aname"] = "_FullZipCode"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._accessCode = None
                    self._address = None
                    self._city = None
                    self._state = None
                    return
            Holder.__name__ = "FullZipCode_Holder"
            self.pyclass = Holder

    class FullZipCodeResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "FullZipCodeResponse"
        def __init__(self, **kw):
            ns = ns1.FullZipCodeResponse_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"FullZipCodeResult"), aname="_FullZipCodeResult", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","FullZipCodeResponse")
            kw["aname"] = "_FullZipCodeResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._FullZipCodeResult = None
                    return
            Holder.__name__ = "FullZipCodeResponse_Holder"
            self.pyclass = Holder

    class ShortZipCode_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "ShortZipCode"
        def __init__(self, **kw):
            ns = ns1.ShortZipCode_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"accessCode"), aname="_accessCode", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"address"), aname="_address", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"city"), aname="_city", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"state"), aname="_state", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","ShortZipCode")
            kw["aname"] = "_ShortZipCode"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._accessCode = None
                    self._address = None
                    self._city = None
                    self._state = None
                    return
            Holder.__name__ = "ShortZipCode_Holder"
            self.pyclass = Holder

    class ShortZipCodeResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "ShortZipCodeResponse"
        def __init__(self, **kw):
            ns = ns1.ShortZipCodeResponse_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"ShortZipCodeResult"), aname="_ShortZipCodeResult", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","ShortZipCodeResponse")
            kw["aname"] = "_ShortZipCodeResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._ShortZipCodeResult = None
                    return
            Holder.__name__ = "ShortZipCodeResponse_Holder"
            self.pyclass = Holder

    class CorrectedAddressHtml_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "CorrectedAddressHtml"
        def __init__(self, **kw):
            ns = ns1.CorrectedAddressHtml_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"accessCode"), aname="_accessCode", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"address"), aname="_address", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"city"), aname="_city", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"state"), aname="_state", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","CorrectedAddressHtml")
            kw["aname"] = "_CorrectedAddressHtml"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._accessCode = None
                    self._address = None
                    self._city = None
                    self._state = None
                    return
            Holder.__name__ = "CorrectedAddressHtml_Holder"
            self.pyclass = Holder

    class CorrectedAddressHtmlResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "CorrectedAddressHtmlResponse"
        def __init__(self, **kw):
            ns = ns1.CorrectedAddressHtmlResponse_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"CorrectedAddressHtmlResult"), aname="_CorrectedAddressHtmlResult", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","CorrectedAddressHtmlResponse")
            kw["aname"] = "_CorrectedAddressHtmlResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._CorrectedAddressHtmlResult = None
                    return
            Holder.__name__ = "CorrectedAddressHtmlResponse_Holder"
            self.pyclass = Holder

    class CorrectedAddressXml_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "CorrectedAddressXml"
        def __init__(self, **kw):
            ns = ns1.CorrectedAddressXml_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"accessCode"), aname="_accessCode", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"address"), aname="_address", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"city"), aname="_city", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), ZSI.TC.String(pname=(ns,"state"), aname="_state", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","CorrectedAddressXml")
            kw["aname"] = "_CorrectedAddressXml"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._accessCode = None
                    self._address = None
                    self._city = None
                    self._state = None
                    return
            Holder.__name__ = "CorrectedAddressXml_Holder"
            self.pyclass = Holder

    class CorrectedAddressXmlResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "CorrectedAddressXmlResponse"
        def __init__(self, **kw):
            ns = ns1.CorrectedAddressXmlResponse_Dec.schema
            TClist = [ns1.USPSAddress_Def(pname=(ns,"CorrectedAddressXmlResult"), aname="_CorrectedAddressXmlResult", minOccurs=1, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","CorrectedAddressXmlResponse")
            kw["aname"] = "_CorrectedAddressXmlResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._CorrectedAddressXmlResult = None
                    return
            Holder.__name__ = "CorrectedAddressXmlResponse_Holder"
            self.pyclass = Holder

    class VersionInfo_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "VersionInfo"
        def __init__(self, **kw):
            ns = ns1.VersionInfo_Dec.schema
            TClist = []
            kw["pname"] = ("http://webservices.eraserver.net/","VersionInfo")
            kw["aname"] = "_VersionInfo"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    return
            Holder.__name__ = "VersionInfo_Holder"
            self.pyclass = Holder

    class VersionInfoResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        schema = "http://webservices.eraserver.net/"
        literal = "VersionInfoResponse"
        def __init__(self, **kw):
            ns = ns1.VersionInfoResponse_Dec.schema
            TClist = [ZSI.TC.String(pname=(ns,"VersionInfoResult"), aname="_VersionInfoResult", minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded"))]
            kw["pname"] = ("http://webservices.eraserver.net/","VersionInfoResponse")
            kw["aname"] = "_VersionInfoResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._VersionInfoResult = None
                    return
            Holder.__name__ = "VersionInfoResponse_Holder"
            self.pyclass = Holder

    class string_Dec(ZSI.TC.String, ElementDeclaration):
        literal = "string"
        schema = "http://webservices.eraserver.net/"
        def __init__(self, **kw):
            kw["pname"] = ("http://webservices.eraserver.net/","string")
            kw["aname"] = "_string"
            class IHolder(str): typecode=self
            kw["pyclass"] = IHolder
            IHolder.__name__ = "_string_immutable_holder"
            ZSI.TC.String.__init__(self, **kw)

    class USPSAddress_Dec(ElementDeclaration):
        literal = "USPSAddress"
        schema = "http://webservices.eraserver.net/"
        def __init__(self, **kw):
            kw["pname"] = ("http://webservices.eraserver.net/","USPSAddress")
            kw["aname"] = "_USPSAddress"
            if ns1.USPSAddress_Def not in ns1.USPSAddress_Dec.__bases__:
                bases = list(ns1.USPSAddress_Dec.__bases__)
                bases.insert(0, ns1.USPSAddress_Def)
                ns1.USPSAddress_Dec.__bases__ = tuple(bases)

            ns1.USPSAddress_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "USPSAddress_Dec_Holder"

# end class ns1 (tns: http://webservices.eraserver.net/)



################################################## 
# RPC_Literal_TestDefinitions_services.py 
# generated by ZSI.generate.wsdl2python
##################################################


from RPC_Literal_TestDefinitions_services_types import *
import urlparse, types
from ZSI.TCcompound import ComplexType, Struct
from ZSI import client
import ZSI

# Locator
class WhiteMesaSoapRpcLitTestSvcLocator:
    SoapTestPortTypeRpc_address = "http://www.whitemesa.net/test-rpc-lit"
    def getSoapTestPortTypeRpcAddress(self):
        return WhiteMesaSoapRpcLitTestSvcLocator.SoapTestPortTypeRpc_address
    def getSoapTestPortTypeRpc(self, url=None, **kw):
        return Soap11TestRpcLitBindingSOAP(url or WhiteMesaSoapRpcLitTestSvcLocator.SoapTestPortTypeRpc_address, **kw)

# Methods
class Soap11TestRpcLitBindingSOAP:
    def __init__(self, url, **kw):
        kw.setdefault("readerclass", None)
        kw.setdefault("writerclass", None)
        # no resource properties
        self.binding = client.Binding(url=url, **kw)
        # no ws-addressing

    # op: echoStruct
    def echoStruct(self, request):
        if isinstance(request, echoStructRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoStructResponse.typecode)
        if isinstance(response, echoStructResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoStructArray
    def echoStructArray(self, request):
        if isinstance(request, echoStructArrayRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoStructArrayResponse.typecode)
        if isinstance(response, echoStructArrayResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoStructAsSimpleTypes
    def echoStructAsSimpleTypes(self, request):
        if isinstance(request, echoStructAsSimpleTypesRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoStructAsSimpleTypesResponse.typecode)
        if isinstance(response, echoStructAsSimpleTypesResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoSimpleTypesAsStruct
    def echoSimpleTypesAsStruct(self, request):
        if isinstance(request, echoSimpleTypesAsStructRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoSimpleTypesAsStructResponse.typecode)
        if isinstance(response, echoSimpleTypesAsStructResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoNestedStruct
    def echoNestedStruct(self, request):
        if isinstance(request, echoNestedStructRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoNestedStructResponse.typecode)
        if isinstance(response, echoNestedStructResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoNestedArray
    def echoNestedArray(self, request):
        if isinstance(request, echoNestedArrayRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoNestedArrayResponse.typecode)
        if isinstance(response, echoNestedArrayResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoStringArray
    def echoStringArray(self, request):
        if isinstance(request, echoStringArrayRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoStringArrayResponse.typecode)
        if isinstance(response, echoStringArrayResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoIntegerArray
    def echoIntegerArray(self, request):
        if isinstance(request, echoIntegerArrayRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoIntegerArrayResponse.typecode)
        if isinstance(response, echoIntegerArrayResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoBoolean
    def echoBoolean(self, request):
        if isinstance(request, echoBooleanRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoBooleanResponse.typecode)
        if isinstance(response, echoBooleanResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

    # op: echoString
    def echoString(self, request):
        if isinstance(request, echoStringRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        kw = {}
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://soapinterop.org/", **kw)
        # no output wsaction
        response = self.binding.Receive(echoStringResponse.typecode)
        if isinstance(response, echoStringResponse.typecode.pyclass) is False:
            raise TypeError, "%s incorrect response type" % (response.__class__)
        return response

_echoStructRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStruct"), ofwhat=[ns0.SOAPStruct_Def(pname="inputStruct", aname="_inputStruct", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructRequest:
    typecode = _echoStructRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStruct = None
        return
echoStructRequest.typecode.pyclass = echoStructRequest

_echoStructResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStructResponse"), ofwhat=[ns0.SOAPStruct_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructResponse:
    typecode = _echoStructResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoStructResponse.typecode.pyclass = echoStructResponse

_echoStructArrayRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStructArray"), ofwhat=[ns0.ArrayOfSOAPStruct_Def(pname="inputStructArray", aname="_inputStructArray", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructArrayRequest:
    typecode = _echoStructArrayRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStructArray = None
        return
echoStructArrayRequest.typecode.pyclass = echoStructArrayRequest

_echoStructArrayResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStructArrayResponse"), ofwhat=[ns0.ArrayOfSOAPStruct_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructArrayResponse:
    typecode = _echoStructArrayResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoStructArrayResponse.typecode.pyclass = echoStructArrayResponse

_echoStructAsSimpleTypesRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStructAsSimpleTypes"), ofwhat=[ns0.SOAPStruct_Def(pname="inputStruct", aname="_inputStruct", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructAsSimpleTypesRequest:
    typecode = _echoStructAsSimpleTypesRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStruct = None
        return
echoStructAsSimpleTypesRequest.typecode.pyclass = echoStructAsSimpleTypesRequest

_echoStructAsSimpleTypesResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStructAsSimpleTypesResponse"), ofwhat=[ZSI.TC.String(pname="outputString", aname="_outputString", encoded=None, minOccurs=1, maxOccurs=1, nillable=True), ZSI.TCnumbers.Iint(pname="outputInteger", aname="_outputInteger", encoded=None, minOccurs=1, maxOccurs=1, nillable=True), ZSI.TCnumbers.FPfloat(pname="outputFloat", aname="_outputFloat", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStructAsSimpleTypesResponse:
    typecode = _echoStructAsSimpleTypesResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._outputString = None
        self._outputInteger = None
        self._outputFloat = None
        return
echoStructAsSimpleTypesResponse.typecode.pyclass = echoStructAsSimpleTypesResponse

_echoSimpleTypesAsStructRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoSimpleTypesAsStruct"), ofwhat=[ZSI.TC.String(pname="inputString", aname="_inputString", encoded=None, minOccurs=1, maxOccurs=1, nillable=True), ZSI.TCnumbers.Iint(pname="inputInteger", aname="_inputInteger", encoded=None, minOccurs=1, maxOccurs=1, nillable=True), ZSI.TCnumbers.FPfloat(pname="inputFloat", aname="_inputFloat", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoSimpleTypesAsStructRequest:
    typecode = _echoSimpleTypesAsStructRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputString = None
        self._inputInteger = None
        self._inputFloat = None
        return
echoSimpleTypesAsStructRequest.typecode.pyclass = echoSimpleTypesAsStructRequest

_echoSimpleTypesAsStructResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoSimpleTypesAsStructResponse"), ofwhat=[ns0.SOAPStruct_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoSimpleTypesAsStructResponse:
    typecode = _echoSimpleTypesAsStructResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoSimpleTypesAsStructResponse.typecode.pyclass = echoSimpleTypesAsStructResponse

_echoNestedStructRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoNestedStruct"), ofwhat=[ns0.SOAPStructStruct_Def(pname="inputStruct", aname="_inputStruct", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoNestedStructRequest:
    typecode = _echoNestedStructRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStruct = None
        return
echoNestedStructRequest.typecode.pyclass = echoNestedStructRequest

_echoNestedStructResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoNestedStructResponse"), ofwhat=[ns0.SOAPStructStruct_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoNestedStructResponse:
    typecode = _echoNestedStructResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoNestedStructResponse.typecode.pyclass = echoNestedStructResponse

_echoNestedArrayRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoNestedArray"), ofwhat=[ns0.SOAPArrayStruct_Def(pname="inputStruct", aname="_inputStruct", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoNestedArrayRequest:
    typecode = _echoNestedArrayRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStruct = None
        return
echoNestedArrayRequest.typecode.pyclass = echoNestedArrayRequest

_echoNestedArrayResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoNestedArrayResponse"), ofwhat=[ns0.SOAPArrayStruct_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoNestedArrayResponse:
    typecode = _echoNestedArrayResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoNestedArrayResponse.typecode.pyclass = echoNestedArrayResponse

_echoStringArrayRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStringArray"), ofwhat=[ns0.ArrayOfstring_Def(pname="inputStringArray", aname="_inputStringArray", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStringArrayRequest:
    typecode = _echoStringArrayRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputStringArray = None
        return
echoStringArrayRequest.typecode.pyclass = echoStringArrayRequest

_echoStringArrayResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStringArrayResponse"), ofwhat=[ns0.ArrayOfstring_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStringArrayResponse:
    typecode = _echoStringArrayResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoStringArrayResponse.typecode.pyclass = echoStringArrayResponse

_echoIntegerArrayRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoIntegerArray"), ofwhat=[ns0.ArrayOfint_Def(pname="inputIntegerArray", aname="_inputIntegerArray", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoIntegerArrayRequest:
    typecode = _echoIntegerArrayRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputIntegerArray = None
        return
echoIntegerArrayRequest.typecode.pyclass = echoIntegerArrayRequest

_echoIntegerArrayResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoIntegerArrayResponse"), ofwhat=[ns0.ArrayOfint_Def(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoIntegerArrayResponse:
    typecode = _echoIntegerArrayResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoIntegerArrayResponse.typecode.pyclass = echoIntegerArrayResponse

_echoBooleanRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoBoolean"), ofwhat=[ZSI.TC.Boolean(pname="inputBoolean", aname="_inputBoolean", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoBooleanRequest:
    typecode = _echoBooleanRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputBoolean = None
        return
echoBooleanRequest.typecode.pyclass = echoBooleanRequest

_echoBooleanResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoBooleanResponse"), ofwhat=[ZSI.TC.Boolean(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoBooleanResponse:
    typecode = _echoBooleanResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoBooleanResponse.typecode.pyclass = echoBooleanResponse

_echoStringRequestTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoString"), ofwhat=[ZSI.TC.String(pname="inputString", aname="_inputString", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStringRequest:
    typecode = _echoStringRequestTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._inputString = None
        return
echoStringRequest.typecode.pyclass = echoStringRequest

_echoStringResponseTypecode = Struct(pname=("http://soapbuilders.org/rpc-lit-test","echoStringResponse"), ofwhat=[ZSI.TC.String(pname="return", aname="_return", encoded=None, minOccurs=1, maxOccurs=1, nillable=True)], pyclass=None, encoded="http://soapbuilders.org/rpc-lit-test")
class echoStringResponse:
    typecode = _echoStringResponseTypecode
    __metaclass__ = pyclass_type
    def __init__(self):
        self._return = None
        return
echoStringResponse.typecode.pyclass = echoStringResponse



#!/usr/bin/env python
import unittest, sys
from ZSI import *


class t8TestCase(unittest.TestCase):
    "Test Any serialize and parse"

    def check_parse_empty_all(self):
        # None
        skip = [TC.FPEnumeration, TC.Enumeration, TC.IEnumeration, TC.List, TC.Integer]
        for typeclass in filter(lambda c: type(c) in [types.ClassType,type] and not issubclass(c, TC.String) and issubclass(c, TC.SimpleType), TC.__dict__.values()):
            if typeclass in skip: continue
            tc = typeclass()
            sw = SoapWriter()
            sw.serialize(None, typecode=tc, typed=True)
            soap = str(sw)
            ps = ParsedSoap(soap)
            parsed = ps.Parse(Any())
            self.assertEqual(None, parsed)

    def check_parse_empty_string(self):
        # Empty String
        typecodes = TC.Any.parsemap.values()
        for tc in filter(lambda c: isinstance(c, TC.String), TC.Any.parsemap.values()):
            sw = SoapWriter()
            sw.serialize("", typecode=tc, typed=True)
            soap = str(sw)
            print 
            print soap
            ps = ParsedSoap(soap)
            parsed = ps.Parse(Any())
            self.assertEqual("", parsed)

    def check_builtins(self):
        typecode = Any(type=True)
        myInt,myLong,myStr,myDate,myFloat = 123,2147483648,\
            u"hello", time.gmtime(), 1.0001
        orig = [myInt,myLong,myStr,myDate,myFloat]

        sw = SoapWriter()
        sw.serialize(orig, typecode=Any())
        print >>sys.stdout, sw
        
        ps = ParsedSoap(str(sw)) 
        parsed = ps.Parse(Any())
        self.assertEqual(len(orig), len(parsed))

        self.assertEqual(myInt, parsed[0])
        self.assertEqual(myLong, parsed[1])
        self.assertEqual(myStr, parsed[2])
        self.assertEqual(myDate[0:6], parsed[3][0:6])
        self.assertEqual(myFloat, parsed[4])
        
        self.assertEqual(type(myInt), type(parsed[0]))
        self.assertEqual(type(myLong), type(parsed[1]))
        self.assertEqual(type(myStr), type(parsed[2]))
        self.assertEqual(tuple, type(parsed[3]))
        self.assertEqual(type(myFloat), type(parsed[4]))

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(t8TestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()





# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.

from ZSI.wstools.Utility import SplitQName
from ZSI.wstools.Namespaces import WSDL
from ZSI import *
from ZSI.client import *
from ZSI.TC import Any
from ZSI.typeinterpreter import BaseTypeInterpreter
import wstools
from wstools.Utility import DOM
from urlparse import urlparse
import weakref

class ServiceProxy:
    """A ServiceProxy provides a convenient way to call a remote web
       service that is described with WSDL. The proxy exposes methods
       that reflect the methods of the remote web service."""

    def __init__(self, wsdl, service=None, port=None, tracefile=None,
                 nsdict=None, transdict=None):
        """
        Parameters:
           wsdl -- WSDLTools.WSDL instance or URL of WSDL.
           service -- service name or index
           port -- port name or index
           tracefile -- 
           nsdict -- key prefix to namespace mappings for serialization
              in SOAP Envelope.
           transdict -- arguments to pass into HTTPConnection constructor.
        """
        self._tracefile = tracefile
        self._nsdict = nsdict or {}
        self._transdict = transdict 
        self._wsdl = wsdl
        if isinstance(wsdl, basestring) is True:
            self._wsdl = wstools.WSDLTools.WSDLReader().loadFromURL(wsdl)

        assert isinstance(self._wsdl, wstools.WSDLTools.WSDL), 'expecting a WSDL instance'
        self._service = wsdl.services[service or 0]
        self.__doc__ = self._service.documentation
        self._port = self._service.ports[port or 0]
        self._name = self._service.name
        
        binding = self._port.getBinding()
        portType = binding.getPortType()
        for item in portType.operations:
            callinfo = wstools.WSDLTools.callInfoFromWSDL(self._port, item.name)
            method = MethodProxy(self, callinfo)
            setattr(self, item.name, method)

    def _call(self, name, *args, **kwargs):
        """Call the named remote web service method."""
        if len(args) and len(kwargs):
            raise TypeError(
                'Use positional or keyword argument only.'
                )

        callinfo = getattr(self, name).callinfo
        soapAction = callinfo.soapAction
        url = callinfo.location
        (protocol, host, uri, query, fragment, identifier) = urlparse(url)
        port = None
        if host.find(':') >= 0:
            host, port = host.split(':')

        if protocol == 'http':
            transport = httplib.HTTPConnection
        elif protocol == 'https':
            transport = httplib.HTTPSConnection
        else:
            raise RuntimeError, 'Unknown protocol %s' %protocol

        binding = Binding(host=host, tracefile=self._tracefile,
                          transport=transport,transdict=self._transdict,
                          port=port, url=url, nsdict=self._nsdict, 
                          soapaction=soapAction,)

        request, response = self._getTypeCodes(callinfo)
        if len(kwargs): args = kwargs
        if request is None:
            request = Any(oname=name)
        binding.Send(url=uri, opname=None, obj=args,
                     nsdict=self._nsdict, soapaction=soapAction, requesttypecode=request)
        return binding.Receive(replytype=response)


    def _getTypeCodes(self, callinfo):
        """Returns typecodes representing input and output messages, if request and/or
           response fails to be generated return None for either or both.
           
           callinfo --  WSDLTools.SOAPCallInfo instance describing an operation.
        """
        prefix = None
        self._resetPrefixDict()
        if callinfo.use == 'encoded':
            prefix = self._getPrefix(callinfo.namespace)
        try:
            requestTC = self._getTypeCode(parameters=callinfo.getInParameters(), literal=(callinfo.use=='literal'))
        except EvaluateException, ex:
            print "DEBUG: Request Failed to generate --", ex
            requestTC = None

        self._resetPrefixDict()
        try:
            replyTC = self._getTypeCode(parameters=callinfo.getOutParameters(), literal=(callinfo.use=='literal'))
        except EvaluateException, ex:
            print "DEBUG: Response Failed to generate --", ex
            replyTC = None
        
        request = response = None
        if callinfo.style == 'rpc':
            if requestTC: request = TC.Struct(pyclass=None, ofwhat=requestTC, pname=callinfo.methodName)
            if replyTC: response = TC.Struct(pyclass=None, ofwhat=replyTC, pname='%sResponse' %callinfo.methodName)
        else:
            if requestTC: request = requestTC[0]
            if replyTC: response = replyTC[0]

        #THIS IS FOR RPC/ENCODED, DOC/ENCODED Wrapper
        if request and prefix and callinfo.use == 'encoded':
            request.oname = '%(prefix)s:%(name)s xmlns:%(prefix)s="%(namespaceURI)s"' \
                %{'prefix':prefix, 'name':request.oname, 'namespaceURI':callinfo.namespace}

        return request, response

    def _getTypeCode(self, parameters, literal=False):
        """Returns typecodes representing a parameter set
           parameters -- list of WSDLTools.ParameterInfo instances representing
              the parts of a WSDL Message.
        """
        ofwhat = []
        for part in parameters:
            namespaceURI,localName = part.type

            if part.element_type:
                #global element
                element = self._wsdl.types[namespaceURI].elements[localName]
                tc = self._getElement(element, literal=literal, local=False, namespaceURI=namespaceURI)
            else:
                #local element
                name = part.name
                typeClass = self._getTypeClass(namespaceURI, localName)
                if not typeClass:
                    tp = self._wsdl.types[namespaceURI].types[localName]
                    tc = self._getType(tp, name, literal, local=True, namespaceURI=namespaceURI)
                else:
                    tc = typeClass(name)
            ofwhat.append(tc)
        return ofwhat

    def _globalElement(self, typeCode, namespaceURI, literal):
        """namespaces typecodes representing global elements with 
             literal encoding.
           typeCode -- typecode representing an element.
           namespaceURI -- namespace 
           literal -- True/False
        """
        if literal:
            typeCode.oname = '%(prefix)s:%(name)s xmlns:%(prefix)s="%(namespaceURI)s"' \
                %{'prefix':self._getPrefix(namespaceURI), 'name':typeCode.oname, 'namespaceURI':namespaceURI}

    def _getPrefix(self, namespaceURI):
        """Retrieves a prefix/namespace mapping.
           namespaceURI -- namespace 
        """
        prefixDict = self._getPrefixDict()
        if prefixDict.has_key(namespaceURI):
            prefix = prefixDict[namespaceURI]
        else:
            prefix = 'ns1'
            while prefix in prefixDict.values():
                prefix = 'ns%d' %int(prefix[-1]) + 1
            prefixDict[namespaceURI] = prefix
        return prefix

    def _getPrefixDict(self):
        """Used to hide the actual prefix dictionary.
        """
        if not hasattr(self, '_prefixDict'):
            self.__prefixDict = {}
        return self.__prefixDict

    def _resetPrefixDict(self):
        """Clears the prefix dictionary, this needs to be done 
           before creating a new typecode for a message 
           (ie. before, and after creating a new message typecode)
        """
        self._getPrefixDict().clear()

    def _getElement(self, element, literal=False, local=False, namespaceURI=None):
        """Returns a typecode instance representing the passed in element.
           element -- XMLSchema.ElementDeclaration instance
           literal -- literal encoding?
           local -- is locally defined?
           namespaceURI -- namespace
        """
        if not element.isElement():
            raise TypeError, 'Expecting an ElementDeclaration'

        tc = None
        elementName = element.getAttribute('name')
        tp = element.getTypeDefinition('type')

        typeObj = None
        if not (tp or element.content):
            nsuriType,localName = element.getAttribute('type')
            typeClass = self._getTypeClass(nsuriType,localName)
            
            typeObj = typeClass(elementName)
        elif not tp:
            tp = element.content

        if not typeObj:
            typeObj = self._getType(tp, elementName, literal, local, namespaceURI)

        minOccurs = int(element.getAttribute('minOccurs'))
        typeObj.optional = not minOccurs

        maxOccurs = element.getAttribute('maxOccurs')
        typeObj.repeatable = (maxOccurs == 'unbounded') or (int(maxOccurs) > 1)

        return typeObj

    def _getType(self, tp, name, literal, local, namespaceURI):
        """Returns a typecode instance representing the passed in type and name.
           tp -- XMLSchema.TypeDefinition instance
           name -- element name
           literal -- literal encoding?
           local -- is locally defined?
           namespaceURI -- namespace
        """
        ofwhat = []
        if not (tp.isDefinition() and tp.isComplex()):
            raise EvaluateException, 'only supporting complexType definition'
        elif tp.content.isComplex():
            if hasattr(tp.content, 'derivation') and tp.content.derivation.isRestriction():
                derived = tp.content.derivation
                typeClass = self._getTypeClass(*derived.getAttribute('base'))
                if typeClass == TC.Array:
                    attrs = derived.attr_content[0].attributes[WSDL.BASE]
                    prefix, localName = SplitQName(attrs['arrayType'])
                    nsuri = derived.attr_content[0].getXMLNS(prefix=prefix)
                    localName = localName.split('[')[0]
                    simpleTypeClass = self._getTypeClass(namespaceURI=nsuri, localName=localName)
                    if simpleTypeClass:
                        ofwhat = simpleTypeClass()
                    else:
                        tp = self._wsdl.types[nsuri].types[localName]
                        ofwhat = self._getType(tp=tp, name=None, literal=literal, local=True, namespaceURI=nsuri)
                else:
                    raise EvaluateException, 'only support soapenc:Array restrictions'
                return typeClass(atype=name, ofwhat=ofwhat, pname=name, childNames='item')
            else:
                raise EvaluateException, 'complexContent only supported for soapenc:Array derivations'
        elif tp.content.isModelGroup():
            modelGroup = tp.content
            for item in modelGroup.content:
                ofwhat.append(self._getElement(item, literal=literal, local=True))

            tc = TC.Struct(pyclass=None, ofwhat=ofwhat, pname=name)
            if not local:
                self._globalElement(tc, namespaceURI=namespaceURI, literal=literal)
            return tc

        raise EvaluateException, 'only supporting complexType w/ model group, or soapenc:Array restriction'
   
    def _getTypeClass(self, namespaceURI, localName):
        """Returns a typecode class representing the type we are looking for.
           localName -- name of the type we are looking for.
           namespaceURI -- defining XMLSchema targetNamespace.
        """
        bti = BaseTypeInterpreter()
        simpleTypeClass = bti.get_typeclass(localName, namespaceURI)
        return simpleTypeClass


class MethodProxy:
    """ """
    def __init__(self, parent, callinfo):
        self.__name__ = callinfo.methodName
        self.__doc__ = callinfo.documentation
        self.callinfo = callinfo
        self.parent = weakref.ref(parent)

    def __call__(self, *args, **kwargs):
        return self.parent()._call(self.__name__, *args, **kwargs)



#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import  unittest, optparse
from ServiceTest import CONFIG_PARSER, DOCUMENT, LITERAL, BROKE, TESTS


def makeTestSuite(document=None, literal=None, broke=None):
    """Return a test suite containing all test cases that satisfy 
    the parameters. None means don't check.

       document -- None, True, False
       literal -- None, True, False
       broke -- None, True, False
    """
    cp = CONFIG_PARSER
    testSections = []
    sections = [\
        'rpc_encoded' , 'rpc_encoded_broke',
        'rpc_literal', 'rpc_literal_broke', 'rpc_literal_broke_interop',
        'doc_literal', 'doc_literal_broke', 'doc_literal_broke_interop',
    ]
    boo = cp.getboolean
    for s,d,l,b in map(\
        lambda sec: \
            (sec, (None,boo(sec,DOCUMENT)), (None,boo(sec,LITERAL)), (None,boo(sec,BROKE))), sections):
        if document in d and literal in l and broke in b:
            testSections.append(s)
        
    suite = unittest.TestSuite()
    for section in testSections:
        moduleList = cp.get(section, TESTS).split()
        for module in  map(__import__, moduleList):
            s = module.makeTestSuite()
            suite.addTest(s)
    return suite


def simple(suite=None):
    section = 'simple'
    moduleList = CONFIG_PARSER.get(section, TESTS).split()
    suite = suite or unittest.TestSuite()
    for module in  map(__import__, moduleList):
        suite.addTest(module.makeTestSuite())
    return suite


def brokeTestSuite():
    return makeTestSuite(broke=True)

def workingTestSuite():
    suite = makeTestSuite(broke=False)
    simple(suite)
    return suite

def docLitTestSuite():
    return makeTestSuite(broke=False, document=True, literal=True)

def rpcLitTestSuite():
    return makeTestSuite(broke=False, document=False, literal=True)

def rpcEncTestSuite():
    return makeTestSuite(broke=False, document=False, literal=False)

def collectTests():
    suite = unittest.TestSuite()
    suite.addTest(brokeTestSuite())
    suite.addTest(workingTestSuite())
    suite.addTest(docLitTestSuite())
    return suite

def main():
    """Gets tests to run from configuration file.
    """
    unittest.TestProgram(defaultTest="workingTestSuite")

if __name__ == "__main__" : main()
    




#!/bin/env python
############################################################################
# Joshua Boverhof<JRBoverhof@lbl.gov>, LBNL
# Monte Goode <MMGoode@lbl.gov>, LBNL
# See Copyright for copyright notice!
###########################################################################
import exceptions, sys, optparse, os, warnings
import ZSI
from ConfigParser import ConfigParser
from ZSI.generate.wsdl2python import WriteServiceModule, ServiceDescription
from ZSI.wstools import WSDLTools, XMLSchema
from ZSI.wstools.logging import setBasicLoggerDEBUG
from ZSI.generate import containers, utility
from ZSI.generate.utility import NCName_to_ClassName as NC_to_CN, TextProtect

"""
wsdl2py

A utility for automatically generating client interface code from a wsdl
definition, and a set of classes representing element declarations and
type definitions.  This will produce two files in the current working 
directory named after the wsdl definition name.

eg. <definition name='SampleService'>
    SampleService.py
    SampleService_types.py

"""
warnings.filterwarnings('ignore', '', exceptions.UserWarning)
def SetDebugCallback(option, opt, value, parser, *args, **kwargs):
    setBasicLoggerDEBUG()
    warnings.resetwarnings()


def SetPyclassMetaclass(option, opt, value, parser, *args, **kwargs):
 	"""set up pyclass metaclass for complexTypes"""
	from ZSI.generate.containers import TypecodeContainerBase, TypesHeaderContainer
	TypecodeContainerBase.metaclass = kwargs['metaclass']
	TypesHeaderContainer.imports.append(\
    		'from %(module)s import %(metaclass)s' %kwargs
    		)



def formatSchemaObject(fname, schemaObj):
    """ In the case of a 'schema only' generation (-s) this creates
        a fake wsdl object that will function w/in the adapters
        and allow the generator to do what it needs to do.
    """
    
    class fake:
        pass

    f = fake()

    if fname.rfind('/'):
        tmp = fname[fname.rfind('/') + 1 :].split('.')
    else:
        tmp = fname.split('.')

    f.name  = tmp[0] + '_' + tmp[1]
    f.types = { schemaObj.targetNamespace : schemaObj }

    return f


def main():
    """ From a wsdl definition create a wsdl object and run the wsdl2python 
        generator.  
    """
    op = optparse.OptionParser()
    
    # Basic options
    op.add_option("-f", "--file",
                  action="store", dest="file", default=None, type="string",
                  help="file to load wsdl from")
    op.add_option("-u", "--url",
                  action="store", dest="url", default=None, type="string",
                  help="URL to load wsdl from")
    op.add_option("-x", "--schema",
                  action="store_true", dest="schema", default=False,
                  help="process just the schema from an xsd file [no services]")
    op.add_option("-d", "--debug",
                  action="callback", callback=SetDebugCallback,
                  help="debug output")
                  
    # WS Options
    op.add_option("-a", "--address",
                  action="store_true", dest="address", default=False,
                  help="ws-addressing support, must include WS-Addressing schema.")
                  
    # pyclass Metaclass 
    op.add_option("-b", "--complexType",
                  action="callback", callback=SetPyclassMetaclass, 
                  callback_kwargs={'module':'ZSI.generate.pyclass', 
                  	'metaclass':'pyclass_type'},
                  help="add convenience functions for complexTypes, including Getters, Setters, factory methods, and properties (via metaclass). *** DONT USE WITH --simple-naming ***")
    
    # Extended generation options
    op.add_option("-e", "--extended",
                  action="store_true", dest="extended", default=False,
                  help="Do Extended code generation.")    
    op.add_option("-z", "--aname",
                  action="store", dest="aname", default=None, type="string",
                  help="pass in a function for attribute name creation")
    op.add_option("-t", "--types",
                  action="store", dest="types", default=None, type="string",
                  help="file to load types from")
    op.add_option("-o", "--output-dir",
                  action="store", dest="output_directory", default=".", type="string",
                  help="file to load types from")
    op.add_option("-s", "--simple-naming",
                  action="store_true", dest="simple_naming", default=False,
                  help="Simplify generated naming.")
    op.add_option("-c", "--clientClassSuffix",
                  action="store", dest="clientClassSuffix", default=None, type="string",
                  help="Suffix to use for service client class (default \"SOAP\")")
    op.add_option("-m", "--pyclassMapModule",
                  action="store", dest="pyclassMapModule", default=None, type="string",
                  help="Python file that maps external python classes to a schema type.  The classes are used as the \"pyclass\" for that type.  The module should contain a dict() called mapping in the format: mapping = {schemaTypeName:(moduleName.py,className) }")
                  
    (options, args) = op.parse_args()

    location = options.file or options.url
    if options.schema is True:
        reader = XMLSchema.SchemaReader(base_url=location)
    else:
        reader = WSDLTools.WSDLReader()

    wsdl = None
    if options.file is not None:
        wsdl = reader.loadFromFile(location)
    elif options.url is not None:
        wsdl = reader.loadFromURL(location)

    if options.simple_naming:
        # Use a different client suffix
        WriteServiceModule.client_module_suffix = "_client"
        # Write messages definitions to a separate file.
        ServiceDescription.separate_messages = True
        # Use more simple type and element class names
        containers.SetTypeNameFunc( lambda n: '%s_' %(NC_to_CN(n)) )
        containers.SetElementNameFunc( lambda n: '%s' %(NC_to_CN(n)) )
        # Don't add "_" to the attribute name (remove when --aname works well)
        containers.ContainerBase.func_aname = lambda instnc,n: TextProtect(str(n))
        # write out the modules with their names rather than their number.
        utility.namespace_name = lambda cls, ns: utility.Namespace2ModuleName(ns)

    if options.clientClassSuffix:
        from ZSI.generate.containers import ServiceContainerBase
        ServiceContainerBase.clientClassSuffix = options.clientClassSuffix

    assert wsdl is not None, 'Must specify WSDL either with --file or --url'
    if options.schema is True:
        wsdl = formatSchemaObject(location, wsdl)

    if options.aname is not None:
        args = options.aname.rsplit('.',1)
        assert len(args) == 2, 'expecting module.function'
        # The following exec causes a syntax error.
        #exec('from %s import %s as FUNC' %(args[0],args[1]))
        assert callable(FUNC),\
            '%s must be a callable method with one string parameter' %options.aname
        from ZSI.generate.containers import TypecodeContainerBase
        TypecodeContainerBase.func_aname = staticmethod(FUNC)

    if options.pyclassMapModule != None:
        mod = __import__(options.pyclassMapModule)
        components = options.pyclassMapModule.split('.')
        for comp in components[1:]:
            mod = getattr(mod, comp)
        extPyClasses = mod.mapping
    else:
        extPyClasses = None
        
    wsm = WriteServiceModule(wsdl, addressing=options.address, do_extended=options.extended, extPyClasses=extPyClasses)
    if options.types != None:
        wsm.setTypesModuleName(options.types)
    if options.schema is False:
         fd = open(os.path.join(options.output_directory, '%s.py' %wsm.getClientModuleName()), 'w+')
         # simple naming writes the messages to a separate file
         if not options.simple_naming:
             wsm.writeClient(fd)
         else: # provide a separate file to store messages to.
             msg_fd = open(os.path.join(options.output_directory, '%s.py' %wsm.getMessagesModuleName()), 'w+')
             wsm.writeClient(fd, msg_fd=msg_fd)
             msg_fd.close()
         fd.close()

    fd = open( os.path.join(options.output_directory, '%s.py' %wsm.getTypesModuleName()), 'w+')
    wsm.writeTypes(fd)
    fd.close()


if __name__ == '__main__':
    main()
